Design<-function(x, fct="line",degree=1,Scal=FALSE){
 if (fct=="line"){ #straight line fit (maximum degree: x^degree)
  X<-rep(1,length(x))
  for (i in 1:degree){
   X<-cbind(X,x^i)
  }
  if (Scal){
#   Scale<-10^(-1*Scal*(1:degree))
   Scale<-rep(1/Scal,degree+1)
   for (i in 1:degree){
    Scale[i+1]<-1/(Scal*sd(X[,i+1]))
   }
   X<-X%*%diag(Scale)
  }else{
   Scale<-rep(1,ncol(X))
  }
 }else if (fct=="poly"){ #polynomial fit with degrees "degree" (not tested)
  X<-matrix(x,nrow=length(x),ncol=length(degree))
  X<-t(apply(X,1,"^",degree))
  if (Scal){
#   Scale<-10^(-1*Scal*degree)
   Scale<-rep(1/Scal,length(degree))
   for (i in 1:length(degree)){
    if (degree[i]!=0){
     Scale[i]<-1/(Scal*sd(X[,i]))
    }
   }
   X<-X%*%diag(Scale)
  }else{
   Scale<-rep(1,ncol(X))
  }
 }
 list(X=X,Scale=Scale)
}

#Bayes fit with 1/sigma prior
BFit1<-function(x,y, fct="line",degree=1,Scal=FALSE){
 D<-Design(x,fct=fct,degree=degree,Scal=Scal)
 SVD<-svd(t(D$X)%*%D$X)
 if (max(SVD$d)/min(SVD$d)<1/.Machine$double.eps){
  V<-solve(t(D$X)%*%D$X) 
 }else{
  CholInv<-solve(chol(t(D$X)%*%D$X))
  V<-CholInv%*%t(CholInv)
#  V<-SVD$v%*%diag(1/SVD$d)%*%t(SVD$u) 
 }
 thet<-V%*%(t(D$X)%*%y)
 alph<--ncol(D$X)/2+length(x)/2
# bet<-(t(y)%*%y-t(thet)%*%(t(X)%*%X)%*%thet)/2
 bet<-(t(y)%*%y-t(y)%*%D$X%*%V%*%t(D$X)%*%y)/2 #more stable
 s<-integrate(ExpInvG,lower=0,upper=Inf,a=alph,b=bet)$value
 #rescaling
 thet<-diag(D$Scale)%*%thet
 V<-diag(D$Scale)%*%V%*%diag(D$Scale)
 list(thet=thet,V=V,alph=alph,bet=bet,s=s,Scale=D$Scale)
}

#expected value of sqrt of inverse gamma
ExpInvG<-function(s,a,b){2*s^2*dinvgamma(s^2,a,b)}

#Bayes fit with 1/sigma prior for several data sets
BFitAll<-function(x,y, fct="line",degree=1,Ind=II,SubInd=IIIndex,Scal=FALSE){
 if (fct=="line"){
  Dim<-degree+1
 }else{
  Dim=length(degree)
 }
 thet<-matrix(0,nrow=Dim,ncol=length(x))
 V<-array(0,dim=c(Dim,Dim,length(x)))
 alph<-rep(0,length=length(x))
 bet<-rep(0,length=length(x))
 s<-rep(0,length=length(x))
 for (i in 1:length(x)){
  Fit<-BFit1(x=x[[i]],y=y[[i]],fct=fct,degree=degree,Scal=Scal)
  thet[,i]<-Fit$thet
  V[,,i]<-Fit$V
  alph[i]<-Fit$alph
  bet[i]<-Fit$bet
  s[i]<-Fit$s
 }
 Fit<-BFit1(x=x[[II]][SubInd],y=y[[II]][SubInd],fct=fct,degree=degree,Scal=Scal) 
 list(thet=thet,V=V,alph=alph,bet=bet,s=s,Scale=Fit$Scale)
}

#linear fit for several data sets
LinFit<-function(x=Data$x,y=Data$y, fct="line",degree=1){
 if (fct=="line"){
  Dim<-degree+1
 }else{
  Dim=length(degree)
 }
 thet<-matrix(0,nrow=Dim,ncol=length(x))
 s<-rep(0,length=length(x))
 for (i in 1:length(x)){
  D<-Design(x[[i]],fct=fct,degree=degree,Scal=FALSE) 
  Fit<-lm(formula=y~D$X+0,data=list(X=D$X,y=y[[i]]))
  thet[,i]<-Fit$coeff
  s[i]<-summary(Fit)$sigma
 }
 list(thet=thet,s=s)
}

PlotFit<-function(Fit,Not=NA,Bayes=0,L,Farb){
 for (j in 1:(ncol(L[[1]])-1)){
  Sum<-0
  screen(j,new=FALSE)
  plot(0,0,xlim=L$Limitsx[,j],ylim=L$Limitsy[,j],col="white",xlab="",ylab="")
  for (i in setdiff(1:length(Fit[[1]][1,]),Not)){
   xx<-seq(from=L$Limitsx[1,j],to=L$Limitsx[2,j],length=100)
   if (Bayes){
    V2<-c(Fit$bet[i]/Fit$alph[i])*Fit$V[,,i]
    nu<-2*Fit$alph[i]
    if (j<(ncol(L[[1]])-1)){
     yy<-dt((xx-Fit$thet[j,i])/sqrt(V2[j,j]),df=nu)/sqrt(V2[j,j])
    }else{
     yy<-2*xx*dinvgamma(xx^2,shape=Fit$alph[i],scale=Fit$bet[i])
#     points(Fit$s[i],0,pch="*",col="gray",cex=2)
    }
    lines(xx,yy,col=Farb[i])
    Sum<-Sum+yy
   }else{
    if (j<(ncol(L[[1]])-1)){
     points(Fit$thet[j,i],0,col=Farb[i])
    }else{
     points(Fit$s[i],0,col=Farb[i])    
    }
   }
  }
  if (Bayes) {
   lines(xx,Sum/sum(Sum,na.rm=TRUE)/diff(xx)[1],col="darkgreen")
  }
 }
}
