#No warranty or guarantee applies to this software , and therefore any users should satisfy themselves that it meets their requirements.

#The subsequent R code shall enable metrologists to calculate and visualize the Bayesian inference for Normal linear regression problems.
#This is done for a show case example, but can easily be adapted to new measurement data by altering the first 15 lines of the code. 
#More information can be found in the publication Klauenberg, K., Wbbeler, G., Mickan, B., Harris, P. M. and Elster, C. (2015), A Guide on Bayesian Normal Linear Regression, Metrologia, ???
#Part of this work has been funded by the EMRP project NEW 04 `Novel mathematical and statistical approaches to uncertainty evaluation'. The EMRP is jointly funded by the EMRP participating countries within EURAMET and the European Union.

####### Input specifications ########
Path<-getwd() #path to data file
File<-"d1-data.txt" #name of data file, default File=d1-data.txt
Path2<-paste(Path,"Pictures",sep="/")#path to r-files and where graphs are saved
EPS<-1 #save graphs in EPS or PNG format
IndPrior<-c(1,3:5) #Indices, which measurement series belong to prior knowledge
IIIndex<-c(4,5,6,19,20,21,25,26,27) #indices define a subset of the current measurements to be used  
Labels<-list(c("Calibration of sonic nozzle",expression(1/sqrt(Re)),expression(paste("effective area in ",mm^2))),
 c(expression(paste("intercept in ",mm^2)),expression(paste("slope in ",mm^2)),expression(paste(sigma," in ", mm^2)),expression(paste("density in ",1/mm^2))))
 #define the labels for the graphs: title, x-axis, y-axis, parameter names, density in 1/units
Farbe<-c("brown","red","blue","green","gold") #define colors to distinguish previous measurement series
Fct<-"line" #shape of linear relation: supported are "line" (default) and "poly"
Degree<-1 #for "line": design matrix X=(1,x,...x^Degree) i.e. Degree is the maximum exponent, default: Degree=1
#for "poly": design matrix X=(x^Degree[1],...x^Degree[p]) i.e. Degree specifies the degrees of the polynomial to occure in design matrix
Scal<-FALSE #Scaling factor, FALSE (default): no scaling, numeric value: scaling matrix S=diag(1/Scal/u(x_i))
xnew<-c(0.001665,0.002,0.003,0.001635,0.00165) #define x-value used for prediction

library(MCMCpack) #for inverse gamma distribution
setwd(Path2)
source("ReadData.r") 
Data<-ReadData(Path1=Path,Path2=Path2,File=File) 
#read data from a file of 3 columns (x, y, Index of measurement series) into a list of x and y values (including previous and current series)
Data$Index<-IndPrior
II<-setdiff(1:length(Data$x),Data$Index)
 
source("Fit.r")
BFit<-BFitAll(x=Data$x,y=Data$y,fct=Fct,degree=Degree,Scal=Scal)
#parameter estimates for non-informative (1/sigma) prior for all series
BFit$thet[1,]
BFit$thet[2,]
BFit$s
BFit$bet/(BFit$alph-1)
LinFit<-LinFit(x=Data$x,y=Data$y,fct=Fct,degree=Degree)   

if (EPS){
 postscript("PriorKn.eps",horizontal=FALSE, width=4, height=4, paper='special',pointsize=11,family="Times") 
}else{
 png("PriorKn.png",width=4,height=4,res=300,units="in",pointsize=8)
}
PlotData(x=Data$x,y=Data$y,Ind=Data$Index,Fit=BFit,fct=Fct,degree=Degree,Farb=Farbe,Label=Labels[[1]])
dev.off()

source("Prior.r")
#Prior<-PriorEstim(BFit,Data$Index) #old method to fit prior (using estimates only)
Prior<-PriorUncEst(BFit,Data$Index)
Prior
alpha0<-(Prior$bet/(Prior$alph-1))^2/(2*Prior$bet^2/(Prior$alph-1)^2/(Prior$alph-2))+2 #IG prior with twice the variability
Prior2<-list(thet=Prior$thet,V=2*Prior$V,VScal=2*Prior$VScal,alph=alpha0,bet=(alpha0-1)*Prior$bet/(Prior$alph-1),Scale=Prior$Scale)

PriorVar<-sqrt(c(Prior$bet/Prior$alph)*diag(Prior$V))
Variab<-1.1*PriorVar%*%t(c(3,-3))
FitVar<-apply(sqrt(matrix(BFit$bet/BFit$alph,nrow=nrow(BFit$thet),ncol=ncol(BFit$thet),byrow=TRUE)*apply(BFit$V,3,diag)),1,min)
FitVariab<-1.1*FitVar%*%t(c(7,-7))
VariabSig<-4*sqrt(Prior$bet/(Prior$alph+1))*dinvgamma(Prior$bet/(Prior$alph+1),shape=Prior$alph,scale=Prior$bet)
Limitsx<-cbind(t(Prior$thet-Variab),c(0,3.5*sqrt(Prior$bet/(Prior$alph-1)/sqrt(Prior$alph-2))),c(min(unlist(Data$x)),max(unlist(Data$x))))
Limitsy<-cbind(rbind(rep(0,length(Variab[,1])),1.6*3*dnorm(0)/apply(cbind(Variab[,1],FitVariab[,1]),1,min)),c(0,VariabSig),
 c(min(unlist(Data$y)),max(unlist(Data$y))))

if (EPS){
 postscript("Prior+PastFit.eps",horizontal=FALSE, width=7, height=4, paper='special',pointsize=11,family="Times") 
}else{
 png("Prior+PastFit.png",width=7,height=4,res=300,units="in",pointsize=8)
}
if (ceiling((length(Prior$thet)+1)/3)==1){
 split.screen(rbind(c(0,0.38,0,1),c(0.31,0.69,0,1),c(0.62,1,0,1)))
}else{
 split.screen(rbind(c(0,0.38,0.38,1),c(0.31,0.69,0.38,1),c(0.62,1,0.38,1),
  c(0,0.38,0,0.62),c(0.31,0.69,0,0.62),c(0.62,1,0,0.62)))
}
PlotFit(BFit,Not=II,Bayes=1,list(Limitsx=Limitsx,Limitsy=Limitsy),Farb="white") #Without series 2
PlotFit(BFit,Not=II,Bayes=0,list(Limitsx=Limitsx,Limitsy=Limitsy),Farb=Farbe) #Without series 2
AddDistr2Plot(Distr=Prior,L=list(Limitsx=Limitsx,Limitsy=Limitsy),Label=Labels[[2]],Farb="black")
screen(2,new=FALSE)
plot(0,0,col="white",axes=FALSE,xlab="",ylab="")
legend(x="topright",legend=c("past knowledge","past estimates","NIG prior"),lty=c(1,0,1),pch=c(NA,1,NA),col=c("darkgreen","gray","black"),bty="n")
close.screen(all=TRUE)
dev.off()

if (EPS){
 postscript("PriorDistr.eps",horizontal=FALSE, width=7, height=4, paper='special',pointsize=11,family="Times") 
}else{
 png("PriorDistr.png",width=7,height=4,res=300,units="in",pointsize=8)
}
if (ceiling((length(Prior$thet)+1)/3)==1){
 split.screen(rbind(c(0,0.38,0,1),c(0.31,0.69,0,1),c(0.62,1,0,1)))
}else{
 split.screen(rbind(c(0,0.38,0.38,1),c(0.31,0.69,0.38,1),c(0.62,1,0.38,1),
  c(0,0.38,0,0.62),c(0.31,0.69,0,0.62),c(0.62,1,0,0.62)))
}
PlotFit(BFit,Not=II,Bayes=0,list(Limitsx=Limitsx,Limitsy=Limitsy),Farb=Farbe) #Without series 2
AddDistr2Plot(Distr=Prior,L=list(Limitsx=Limitsx,Limitsy=Limitsy),Farb="black",Label=Labels[[2]])
legend(x="topright",legend=c("past estimates","NIG prior"),lty=c(0,1),pch=c(1,NA),col=c("gray","black"),bty="n")
close.screen(all=TRUE)
dev.off()

if (EPS){
 postscript("PriorRegr.eps",horizontal=FALSE, width=4, height=4, paper='special',pointsize=11,family="Times") 
}else{
 png("PriorRegr.png",width=4,height=4,res=300,units="in",pointsize=8)
}
PlotData(x=Data$x,y=Data$y,Ind=Data$Index,Fit=BFit,Farb=Farbe,Label=Labels[[1]],fct=Fct,degree=Degree)
AddRegr(Regr=Prior,L=list(Limitsx=Limitsx,Limitsy=Limitsy),fct=Fct,degree=Degree,Farb="black")
dev.off()

if (EPS){
 postscript("Data.eps",horizontal=FALSE, width=4, height=4, paper='special',pointsize=11,family="Times") 
}else{
 png("Data.png",width=4,height=4,res=300,units="in",pointsize=8)
}
PlotData(x=Data$x,y=Data$y,Ind=II,Fit=BFit,Farb=Farbe,SubInd=IIIndex,Label=Labels[[1]])
dev.off()

source("Post.r")
Post<-PostFit(x=Data$x[[II]],y=Data$y[[II]],SubInd=IIIndex,Prior=Prior,fct=Fct,degree=Degree,Scale=BFit$Scale)
Post

if (EPS){
 postscript("PostDistr.eps",horizontal=FALSE, width=7, height=4, paper='special',pointsize=11,family="Times") 
}else{
 png("PostDistr.png",width=7,height=4,res=300,units="in",pointsize=8)
}
if (ceiling((length(Prior$thet)+1)/3)==1){
 split.screen(rbind(c(0,0.38,0,1),c(0.31,0.69,0,1),c(0.62,1,0,1)))
}else{
 split.screen(rbind(c(0,0.38,0.38,1),c(0.31,0.69,0.38,1),c(0.62,1,0.38,1),
  c(0,0.38,0,0.62),c(0.31,0.69,0,0.62),c(0.62,1,0,0.62)))
}
AddDistr2Plot(Distr=Prior,L=list(Limitsx=Limitsx,Limitsy=Limitsy),Farb="black",Label=Labels[[2]])
AddDistr2Plot(Distr=Post,L=list(Limitsx=Limitsx,Limitsy=Limitsy),Labels=0)
screen(2,new=FALSE)
plot(0,0,col="white",axes=FALSE,xlab="",ylab="")
legend(x="top",legend=c("NIG prior ","NIG posterior"),lty=c(1,1),col=c("black","red"),bty="n")
close.screen(all=TRUE)
dev.off()

Estimates(Post)
Pred<-Prediction(Post,xnew=xnew,fct=Fct,degree=Degree)
Pred

if (EPS){
 postscript("PostRegr.eps",horizontal=FALSE, width=4, height=4, paper='special',pointsize=11,family="Times") 
}else{
 png("PostRegr.png",width=4,height=4,res=300,units="in",pointsize=8)
}
PlotData(x=Data$x,y=Data$y,Ind=II,Fit=BFit,Farb=Farbe,SubInd=IIIndex,Label=Labels[[1]])
AddRegr(Regr=Post,L=list(Limitsx=Limitsx,Limitsy=Limitsy),Farb="red",fct=Fct,degree=Degree)
lines(rep(xnew[2],2),Pred$CI[2,],col="red")
points(xnew[2],Pred$ynew[2],col="red",pch=8)
legend(x="topright",legend=c("Data","Posterior regression","Posterior prediction"),lty=c(0,1,1),pch=c(1,NA,8),lwd=c(1,2,1),col="red",bty="n")
dev.off()

#Sensitivity
Post2<-PostFit(x=Data$x[[II]],y=Data$y[[II]],SubInd=IIIndex,Prior=Prior2,fct=Fct,degree=Degree,Scale=BFit$Scale)
Post3<-BFit1(x=Data$x[[II]][IIIndex],y=Data$y[[II]][IIIndex],fct=Fct,degree=Degree,Scal=Scal)

if (EPS){
 postscript("SensiDistr.eps",horizontal=FALSE, width=7, height=4, paper='special',pointsize=11,family="Times") 
}else{
 png("SensiDistr.png",width=7,height=4,res=300,units="in",pointsize=8)
}
if (ceiling((length(Prior$thet)+1)/3)==1){
 split.screen(rbind(c(0,0.38,0,1),c(0.31,0.69,0,1),c(0.62,1,0,1)))
}else{
 split.screen(rbind(c(0,0.38,0.38,1),c(0.31,0.69,0.38,1),c(0.62,1,0.38,1),
  c(0,0.38,0,0.62),c(0.31,0.69,0,0.62),c(0.62,1,0,0.62)))
}
AddDistr2Plot(Distr=Post2,L=list(Limitsx=Limitsx,Limitsy=Limitsy),Farb="purple",Label=Labels[[2]],Lwd=3)
AddDistr2Plot(Distr=Post3,L=list(Limitsx=Limitsx,Limitsy=Limitsy),Farb="orchid",Labels=0,Lwd=2)
AddDistr2Plot(Distr=Prior,L=list(Limitsx=Limitsx,Limitsy=Limitsy),Farb="black",Labels=0)  
AddDistr2Plot(Distr=Post,L=list(Limitsx=Limitsx,Limitsy=Limitsy),Labels=0)  
AddDistr2Plot(Distr=Prior2,L=list(Limitsx=Limitsx,Limitsy=Limitsy),Farb="gray",Labels=0)
screen(2,new=FALSE)
plot(0,0,col="white",axes=FALSE,xlab="",ylab="")
legend(x="top",legend=c("NIG prior","Prior A","NIG posterior","Post. (Prior A)","Post. (Prior C)"),
 lty=1,col=c("black","gray","red","purple","orchid"),lwd=c(1,1,1,3,2),bty="n")  
close.screen(all=TRUE)
dev.off()

Estimates(Post2)
Estimates(Post3)
Pred2<-Prediction(Post2,xnew=xnew,fct=Fct,degree=Degree)
Pred3<-Prediction(Post3,xnew=xnew,fct=Fct,degree=Degree)
Pred2
Pred3
#comparison at x=0.001665, 0.002, 0.003
Pred2$y-Pred$y
Pred3$y-Pred$y
(apply(Pred2$CI,1,diff)-apply(Pred$CI,1,diff))/apply(Pred$CI,1,diff)*100
(apply(Pred3$CI,1,diff)-apply(Pred$CI,1,diff))/apply(Pred$CI,1,diff)*100


if (EPS){
 postscript("SensiRegr.eps",horizontal=FALSE, width=7, height=4, paper='special',pointsize=11,family="Times") 
}else{
 png("SensiRegr.png",width=7,height=4,res=300,units="in",pointsize=8)
}
par(mfrow=c(1,2))
PlotData(x=Data$x,y=Data$y,Ind=II,Fit=BFit,Farb=Farbe,SubInd=IIIndex,Label=Labels[[1]])
AddRegr(Regr=Post3,L=list(Limitsx=Limitsx,Limitsy=Limitsy),Farb="orchid",fct=Fct,degree=Degree,Lwd=2)
lines(rep(xnew[2],2),Pred3$CI[2,],col="orchid",lwd=2)      
legend(x="bottomleft",legend=c("Post. regr.","Post. regr. (Prior A)","Post. regr. (Prior C)"),
 lty=1,lwd=c(1,3,2),col=c("red","purple","orchid"),box.lty=0,bg="white",inset=.004)
AddRegr(Regr=Post2,L=list(Limitsx=Limitsx,Limitsy=Limitsy),Farb="purple",fct=Fct,degree=Degree,Lwd=3)
AddRegr(Regr=Post,L=list(Limitsx=Limitsx,Limitsy=Limitsy),Farb="red",fct=Fct,degree=Degree,Lwd=1)           
lines(rep(xnew[2],2),Pred2$CI[2,],col="purple",lwd=3)
lines(rep(xnew[2],2),Pred$CI[2,],col="red",lwd=1) 
points(xnew[2],Pred$ynew[2],col="red",pch="*",cex=2)
legend(x="topright",legend=c("Data","Post. pred."),lty=c(0,1),pch=c(1,8),lwd=1,col=c("red","gray"),bty="n",xjust=1)
#plot(0,0,col="white",axes=FALSE,xlab="",ylab="") #to plot only right hand side
PlotData(x=Data$x,y=Data$y,Ind=II,Fit=BFit,Farb=Farbe,SubInd=IIIndex,FullRange=FALSE,Label=Labels[[1]])
AddRegr(Regr=Post3,L=list(Limitsx=Limitsx,Limitsy=Limitsy),Farb="orchid",fct=Fct,degree=Degree,Lwd=2)
AddRegr(Regr=Post2,L=list(Limitsx=Limitsx,Limitsy=Limitsy),Farb="purple",fct=Fct,degree=Degree,Lwd=3)
AddRegr(Regr=Post,L=list(Limitsx=Limitsx,Limitsy=Limitsy),Farb="red",fct=Fct,degree=Degree,Lwd=1)  
lines(rep(xnew[2],2),Pred3$CI[2,],col="orchid",lwd=2)
lines(rep(xnew[2],2),Pred2$CI[2,],col="purple",lwd=3)
lines(rep(xnew[2],2),Pred$CI[2,],col="red")  
points(xnew[2],Pred$ynew[2],col="red",pch="*",cex=2)
dev.off()

#Model Check
#prob of the intercept exceeding 77.148
#pt((77.148-Post$thet[1])/sqrt(c(Post$bet/Post$alph)*Post$V[1,1]),df=2*Post$alp,lower.tail=FALSE) 
#prob of the slope beeing negative
#pt((0-Post$thet[2])/sqrt(c(Post$bet/Post$alph)*Post$V[2,2]),df=2*Post$alp,lower.tail=FALSE)
xpred<-Data$x[[II]][setdiff(1:length(Data$x[[II]]),IIIndex)]
PredAll<-Prediction(Post,xnew=unique(xpred),fct=Fct,degree=Degree)

if (EPS){
 postscript("PostPred.eps",horizontal=FALSE, width=4, height=4, paper='special',pointsize=11,family="Times") 
}else{
 png("PostPred.png",width=4,height=4,res=300,units="in",pointsize=8)
}
Farb<-"red"
PlotData(x=Data$x,y=Data$y,Ind=II,Fit=BFit,Farb=c(NA,"purple"),SubInd=setdiff(1:length(Data$x[[II]]),IIIndex),Label=Labels[[1]])
for (i in 1:length(unique(xpred))){
 lines(rep(unique(xpred)[i],2),PredAll$CI[i,],col=Farb) 
 points(unique(xpred)[i],PredAll$ynew[i],col=Farb,pch="*")
}
legend(x="bottomleft",legend=c("Additional data","Post. prediction"),lty=c(0,1),col=c("purple","red"),pch=c(1,8),bty="n")
dev.off()

xnew<-Data$x[[II]][IIIndex]
pred2<-Prediction(Post,xnew,fct=Fct,degree=Degree)

if (EPS){
 postscript("Residuals.eps",horizontal=FALSE, width=4, height=4, paper='special',pointsize=11,family="Times") 
}else{
 png("Residuals.png",width=4,height=4,res=300,units="in",pointsize=8)
}
PlotData(x=list(xnew),y=list(c(Post$res)),Ind=1,SubInd=1:length(IIIndex),FullRange=FALSE,
 YFac=max(pred2$CI[,2]-pred2$ynew)/max(Post$res)-0.45,Offset=0.015,Fit=BFit,Farb="red",Label=c("",expression(1/sqrt(Re)),expression(paste("(unstandardized) residuals in ",mm^2))))
for (i in 1:length(unique(xnew))){
 lines(rep(unique(xnew)[i],2),pred2$CI[i,]-pred2$ynew[i],col="red")
}
legend(x="topleft",legend=c("realized residuals","predicted residuals"),col="red",lty=c(0,1),pch=c(1,NA),bty="n")
dev.off()
