PostFit<-function(x,y,SubInd, Prior, fct="line",degree=1,Scale){
 D<-Design(x=x[SubInd],fct=fct,degree=degree,Scal=FALSE)
 X<-D$X%*%diag(Scale)
 SVD<-svd(Prior$VScal)
 if (max(SVD$d)/min(SVD$d)<1/.Machine$double.eps){  #numerically instable inversion (badly conditioned)
  V0inv<-solve(Prior$VScal) 
 }else if (any(SVD$d<.Machine$double.eps)){ #prior cov matrix is rank deficient
  InvSV<-1/SVD$d
  InvSV[which(SVD$d<sqrt(.Machine$double.eps))]<-0
  V0inv<-SVD$v%*%diag(InvSV)%*%t(SVD$u) 
 }else{
  CholInv<-solve(chol(Prior$VScal))
  V0inv<-CholInv%*%t(CholInv)
 }
 SVD<-svd(V0inv+t(X)%*%X)
 if (max(SVD$d)/min(SVD$d)<1/.Machine$double.eps){
  V1<-solve(V0inv+t(X)%*%X)
 }else{
  CholInv<-solve(chol(V0inv+t(X)%*%X))
  V1<-CholInv%*%t(CholInv)
 }
 thetP<-diag(1/Scale)%*%Prior$thet
 theta1<-V1%*%(V0inv%*%thetP+t(X)%*%y[SubInd])
 alpha1<-Prior$alph+length(SubInd)/2
# beta1<-Prior$bet+(t(thetP)%*%V0inv%*%thetP+t(y[SubInd])%*%y[SubInd]-t(theta1)%*%(V0inv+t(X)%*%X)%*%theta1)/2
 yC<-y[SubInd]-mean(y[SubInd])  
 thetC<-thetP-solve(t(X)%*%X)%*%t(X)%*%rep(mean(y[SubInd]),length(SubInd))
 theta1C<-V1%*%(V0inv%*%thetC+t(X)%*%yC)
# beta1<-Prior$bet+(t(thetP)%*%V0inv%*%thetP+t(y[SubInd])%*%y[SubInd]-t(V0inv%*%thetP+t(X)%*%y[SubInd])%*%V1%*%(V0inv%*%thetP+t(X)%*%y[SubInd]))/2
 beta1<-Prior$bet+(t(thetC)%*%V0inv%*%thetC+t(yC)%*%yC-t(theta1C)%*%(V0inv+t(X)%*%X)%*%theta1C)/2 #use centered y for stability
 s<-integrate(ExpInvG,lower=0,upper=Inf,a=alpha1,b=beta1)$value
 resid<-(y[SubInd]-X%*%theta1)
 theta1<-diag(Scale)%*%theta1
 V1<-diag(Scale)%*%V1%*%diag(Scale)
 list(thet=theta1,V=V1,alph=alpha1,bet=beta1,s=s,res=resid)
}

ExpInvG<-function(s,a,b){2*s^2*dinvgamma(s^2,a,b)}
#VarInvG<-function(s,a,b,offset=0){2*s^3*dinvgamma((s-offset)^2,a,b)}
VarInvG<-function(s,a,b,offs=0){2*s*(s-offs)^2*dinvgamma(s^2,a,b)}
 
Estimates<-function(Distr=Post,Credib=0.95){
 CIlow<-(1-Credib)/2
 CIhigh<-1-CIlow
 nu<-2*Distr$alph
 V2<-c(Distr$bet/Distr$alph)*Distr$V
 utheta<-sqrt(diag(nu/(nu-2)*V2))
 CI<-matrix(0,nrow=nrow(V2),ncol=2)
 for (i in 1:nrow(V2)){
  CI[i,]<-qt(p=c(CIlow,CIhigh),nu)*sqrt(V2[i,i])+Distr$thet[i]
 }
 sigmahat<-integrate(ExpInvG,lower=0,upper=Inf,a=Distr$alph,b=Distr$bet)$value
 usigma<-sqrt(integrate(VarInvG,lower=0,upper=Inf,a=Distr$alph,b=Distr$bet,offs=sigmahat,rel.tol = 1/1000*.Machine$double.eps^0.25)$value)
# usigma<-sqrt(integrate(VarInvG,lower=0,upper=Inf,a=Distr$alph,b=Distr$bet)$value-sigmahat^2) #less stable
 sigma2<-Distr$bet/(Distr$alph-1) #mean of variance (not sigma)
 usigma2<-Distr$bet/(Distr$alph-1)/sqrt(Distr$alph-2)
 Sample<-sqrt(rinvgamma(10000000,shape=Distr$alph,scale=Distr$bet))
 CIs<-rbind(quantile(Sample,prob=c(CIlow,CIhigh)),quantile((Sample)^2,prob=c(CIlow,CIhigh)))
 list(thet=cbind(Distr$thet,utheta,CI), sig=cbind(c(sigmahat, sigma2),c(usigma, usigma2),CIs))
}

#Residuals<-function(x,y,Distr,){
#}

Prediction<-function(Distr=Post,xnew=0,fct="line",degree=1,Credib=0.95){
 CIlow<-(1-Credib)/2
 CIhigh<-1-CIlow
 D<-Design(x=xnew,fct=fct,degree=degree,Scal=FALSE)
 ynew<-D$X%*%Distr$thet
 nu<-2*Distr$alph
 V3<-c(Distr$bet/Distr$alph)*(diag(length(xnew))+D$X%*%Distr$V%*%t(D$X))
 uy<-sqrt(diag(nu/(nu-2)*V3))
 CI<-matrix(0,nrow=nrow(V3),ncol=2)
 for (i in 1:nrow(V3)){
  CI[i,]<-qt(p=c(CIlow,CIhigh),nu)*sqrt(V3[i,i])+ynew[i] 
 }
 list(ynew=ynew,uy=uy,CI=CI,V=V3,nu=nu)
}